#!/usr/bin/perl

# rip-utils
#
# Copyright (C) 2001-2002 Mikol Graves
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA.

# ---------------------------------------------------------------------------
package tbx::Album;
# ---------------------------------------------------------------------------

use tbx::Track();

sub new {
  my $this = shift();
  my $class = ref($this) || $this;
  my $self = {};
  bless($self, $class);
  $self->initialize(@_);
  return $self;
}

sub initialize {
  my $this = shift();
  $this->{id} = shift();
  $this->{artist} = shift();
  $this->{title}  = shift();
  $this->{cat} = shift();
  $this->{eid} = shift();
  $this->{tracks} = 0;
  $this->{$this->{tracks}} = $this;
}

sub add_track {
  my $this = shift();
  $this->{tracks}++;
  $this->{$this->{tracks}} = new tbx::Track($this->{tracks}, @_);
  return $this->{$this->{tracks}};
}

sub freedb_id {
  my ($this, $h) = @_;
  $this->{id} = $h if defined $h;
  return $this->{id};
}

sub freedb_cat {
  my ($this, $s) = @_;
  $this->{cat} = $s if defined $s;
  return $this->{cat};
}

sub freedb_eid {
  my ($this, $h) = @_;
  $this->{eid} = $h if defined $h;
  return $this->{eid};
}

sub genre {
  my ($this, $s) = @_;
  $this->{genre} = $s if defined $s;
  return $this->{genre};
}

sub year {
  my ($this, $s) = @_;
  $this->{year} = $s if defined $s;
  return $this->{year};
}

sub title {
  my ($this, $s) = @_;
  $this->{title} = $s if defined $s;
  return $this->{title};
}

sub artist {
  my ($this, $s) = @_;
  $this->{artist} = $s if defined $s;
  return $this->{artist};
}

sub length {
  my ($this, $i) = @_;
  $this->{length} = $i if defined $i;
  return $this->{length};
}

sub number_of_tracks {
  my ($this) = @_;
  return $this->{tracks};
}

# range -----------------------------------------------------------------------

# Given a string range, return an array of tracks that the range covers. A
# valid range looks something like '1,4-7,10'.

# -----------------------------------------------------------------------------
sub range {
  my ($this, $range) = @_;
  $range =~ s/\s//g;                            # remove whitespace

  if ($range =~ /^all$/) {                      # $range is a keyword.
    $range = 0 . "-" . 
      $this->{$this->number_of_tracks()}->number();
  }
  if ($range =~ /^none$/) {                     # $range is a keyword.
    $range = "";
  }

  my @set = ();
  my @t = split(",", $range);             
  for my $t (@t) {
    if ($t =~ /-/) {                            # $t is a range.
      my @r = split("-", $t);                   # 
      if (scalar(@r) == 2 &&                    # A range should have 2 values.
          $r[1] <= $this->{$this->number_of_tracks()}->number() && 
          $r[0] < $r[1])                        # The 2nd cannot be > # tracks.
      {                                         # The 1st cannot be <= the 2nd.
        for(my $i = $r[0]; $i <= $r[1]; $i++) {
          $set[scalar(@set)] = $i;
        }
      } else {
        return;
      }
    } elsif ($t =~ /^\d+$/) {                   # $t is a digit.
      if ($t <= $this->{$this->number_of_tracks()}->number()) {
        $set[scalar(@set)] = $t;                # 0 < $t < # tracks.
      } else {
        return;
      }
    } else {                                    # $t is not a digit or a range.
      return;
    }
  }

  return (@set);
}

sub DESTROY {
  my $this = shift;
  $this->{0} = 0;
}

1;